@@ -1,37 +1,39 @@
 import os
 import numpy as np
 from osgeo import ogr, osr
+import importlib
+
 from shapely.ops import polygonize
 from pyflowline.algorithms.auxiliary.gdal_functions import  calculate_angle_betwen_vertex_normal
 from pyflowline.algorithms.auxiliary.gdal_functions import calculate_polygon_area
-from pyflowline.algorithms.auxiliary.find_index_in_list import find_list_in_list ,find_vertex_in_list
+from pyflowline.algorithms.auxiliary.find_index_in_list import find_list_in_list 
+
+iFlag_cython = importlib.util.find_spec("cython") 
+if iFlag_cython is not None:
+    from pyflowline.algorithms.cython.kernel import find_vertex_in_list
+else:
+    from pyflowline.algorithms.auxiliary.find_vertex_in_list import find_vertex_in_list
+
 
 def calculate_area_of_difference_raw(sFilename_a, sFilename_b):
     #not yet supported
     return
 
-def calculate_area_of_difference_simplified(aFlowline_in, aVertex_all_in, \
-     sFilename_output_in):
-
+def calculate_area_of_difference_simplified(aFlowline_in, aVertex_all_in, sFilename_output_in):   
     if os.path.exists(sFilename_output_in): 
-        #delete it if it exists
         os.remove(sFilename_output_in)
         pass
 
-    pDriver_geojson = ogr.GetDriverByName( "GeoJSON")
-
+    pDriver_geojson = ogr.GetDriverByName("GeoJSON")
     nFlowline = len(aFlowline_in)
     nVeretx = len(aVertex_all_in)
-
-    #rebuild
+    #rebuild index
     for i  in range(nFlowline):
         aFlowline_in[i].lFlowlineID = i
     for i  in range(nVeretx):
         aVertex_all_in[i].lVertexID = i
 
-    for i  in range(nVeretx):
-    #for n, v in enumerate(aVertex_all_in):
-    
+    for i  in range(nVeretx):    
         for j in range(nFlowline):
             if aVertex_all_in[i] == aFlowline_in[j].pVertex_start:
                 aFlowline_in[j].pVertex_start.lVertexID = i
@@ -60,10 +62,8 @@ def calculate_area_of_difference_simplified(aFlowline_in, aVertex_all_in, \
         the angles between the adjacent contacts at a side (either `stop` or
         `start`) and then chooses the right-branching contact, and the opposite
         side for the next contact.
-
         For example, if we are at `edge_0` and `edge_2` is the right-branching contact,
         and the `start` side of `edge_2` meets `edge_0`, then the next side will be `stop`.
-
         """
 
         nEdge = pFlowline_in.nEdge
@@ -72,15 +72,11 @@ def calculate_area_of_difference_simplified(aFlowline_in, aVertex_all_in, \
             #normal direction
             pVertex_stop = pFlowline_in.pVertex_end
             pEdge = pFlowline_in.aEdge[nEdge-1]
-
             x1 = pEdge.pVertex_start.dLongitude_degree
             y1 = pEdge.pVertex_start.dLatitude_degree
             x2 = pEdge.pVertex_end.dLongitude_degree
-            y2 = pEdge.pVertex_end.dLatitude_degree
-            
-           
+            y2 = pEdge.pVertex_end.dLatitude_degree                       
             iIndex_right = -1
-
             n = len(pFlowline_in.aFlowlineID_end_start) + len(pFlowline_in.aFlowlineID_end_end)
             if n == 0:
                 iFlag_exist = 0 
@@ -96,7 +92,7 @@ def calculate_area_of_difference_simplified(aFlowline_in, aVertex_all_in, \
                         nEdge2 = pFlowline_dummy.nEdge
                         pEdge2 = pFlowline_dummy.aEdge[0]
                         pVertex_dummy = pEdge2.pVertex_end
-                    else:                        #revered
+                    else:  #revered
                         nEdge2 = pFlowline_dummy.nEdge
                         pEdge2 = pFlowline_dummy.aEdge[nEdge2-1]
                         pVertex_dummy = pEdge2.pVertex_start
@@ -112,7 +108,7 @@ def calculate_area_of_difference_simplified(aFlowline_in, aVertex_all_in, \
                         nEdge2 = pFlowline_dummy.nEdge
                         pEdge2 = pFlowline_dummy.aEdge[0]
                         pVertex_dummy = pEdge2.pVertex_end
-                    else:                        #revered
+                    else:    #revered
                         nEdge2 = pFlowline_dummy.nEdge
                         pEdge2 = pFlowline_dummy.aEdge[nEdge2-1]
                         pVertex_dummy = pEdge2.pVertex_start
@@ -122,8 +118,7 @@ def calculate_area_of_difference_simplified(aFlowline_in, aVertex_all_in, \
                     y3 = pVertex_dummy.dLatitude_degree
                     angle_dummy = calculate_angle_betwen_vertex_normal( x1, y1, x2, y2, x3, y3  )                    
                     aAngle.append(angle_dummy)
-                    
-
+                  
                 #mini
                 dummy = pFlowline_in.aFlowlineID_end_start + pFlowline_in.aFlowlineID_end_end
                 if iFlag_rightleft ==0 :
@@ -147,7 +142,6 @@ def calculate_area_of_difference_simplified(aFlowline_in, aVertex_all_in, \
                 iFlag_exist = 1
                 return iFlag_exist, iFlag_reverse_new, pFlowline_out, pVertex_stop_out
 
-
         else:
             #rever direction
             pVertex_stop = pFlowline_in.pVertex_start
@@ -225,21 +219,18 @@ def calculate_area_of_difference_simplified(aFlowline_in, aVertex_all_in, \
     def walk_cycle(iFlag_rightleft, iFlag_reverse, pFlowline_in, pVertex_origin, aFlowline_list, aVertex_list ):
         iFlag_loop = 1
         if iFlag_reverse ==0:
-
             iFlag_right = pFlowline_in.iFlag_right
             if iFlag_right == 1:
                 iFlag_loop = 0
                 return iFlag_loop, aFlowline_list
-            else:
-                
+            else:                
                 iFlag_exist, iFlag_reverse_new, pFlowline_next, pVertex_stop  = get_next_branch(iFlag_rightleft, 0, pFlowline_in)
                 if iFlag_exist ==1:
                     aFlowline_list.append(pFlowline_next.lFlowlineID)
-                    if (pVertex_stop == pVertex_origin):
-                        #a loop is finished
+                    if (pVertex_stop == pVertex_origin):                        
                         iFlag_loop = 1
                         return iFlag_loop, aFlowline_list
-                        pass
+                      
                     else:
                         iFlag_vertex_exist, index_dummy = find_vertex_in_list(aVertex_list, pVertex_stop)
                         if iFlag_vertex_exist ==1:
@@ -261,15 +252,10 @@ def calculate_area_of_difference_simplified(aFlowline_in, aVertex_all_in, \
             if iFlag_right == 1:
                 iFlag_loop = 0
                 return iFlag_loop, aFlowline_list
-            else:
-                
-                iFlag_exist, iFlag_reverse_new, pFlowline_next, pVertex_stop = get_next_branch(iFlag_rightleft,1, pFlowline_in)
-                
-                if iFlag_exist ==1:
-                    #aFlowline_in[pFlowline_in.lFlowlineID].iFlag_right=1
-
+            else:                
+                iFlag_exist, iFlag_reverse_new, pFlowline_next, pVertex_stop = get_next_branch(iFlag_rightleft,1, pFlowline_in)                
+                if iFlag_exist ==1:       
                     aFlowline_list.append(pFlowline_next.lFlowlineID)
-
                     if (pVertex_stop == pVertex_origin):
                         #a loop is finished
                         iFlag_loop = 1
@@ -289,12 +275,10 @@ def calculate_area_of_difference_simplified(aFlowline_in, aVertex_all_in, \
                 else:
                     iFlag_loop = 0
                     return iFlag_loop, aFlowline_list
-
                 
         return iFlag_loop, aFlowline_list
 
     aList_all = list()
-
     for i in range(0,nFlowline):
         pFlowline_in = aFlowline_in[i]
         aFlowline_list=list()
@@ -306,24 +290,19 @@ def calculate_area_of_difference_simplified(aFlowline_in, aVertex_all_in, \
             if iFlag == 1:
                 pass
             else:
-                aList_all.append(aFlowline_list)
-                
-
+                aList_all.append(aFlowline_list)            
     
     pDataset = pDriver_geojson.CreateDataSource(sFilename_output_in) 
     pSpatial_reference_gcs = osr.SpatialReference()  
     pSpatial_reference_gcs.ImportFromEPSG(4326)    # WGS84 lat/lon
     pSpatial_reference_gcs.SetAxisMappingStrategy(osr.OAMS_TRADITIONAL_GIS_ORDER)
-
     pLayer = pDataset.CreateLayer('aod', pSpatial_reference_gcs, ogr.wkbPolygon)
     # Add one attribute
     pLayer.CreateField(ogr.FieldDefn('id', ogr.OFTInteger64)) #long type for high resolution
-
     pLayerDefn = pLayer.GetLayerDefn()
     pFeature = ogr.Feature(pLayerDefn)    
     lCellID = 0
     dArea = 0.0
-
     #shapely method, but not complete
     for n in range(0,len(aList_all)):
         aFlowline_list = aList_all[n]
@@ -338,12 +317,11 @@ def calculate_area_of_difference_simplified(aFlowline_in, aVertex_all_in, \
                 aCoords_gcs[k,0] = pFlowline.aVertex[k].dLongitude_degree
                 aCoords_gcs[k,1] = pFlowline.aVertex[k].dLatitude_degree
             
-            aCoords_gcs= tuple(j for j in aCoords_gcs) #np.array(aCoords_gcs)    
+            aCoords_gcs= tuple(j for j in aCoords_gcs)  
             aFlowline_polygon.append(aCoords_gcs)   
 
         dummy = polygonize(aFlowline_polygon)
-        aPolygon_out =  list(dummy) 
-  
+        aPolygon_out =  list(dummy)   
         for po in aPolygon_out:
             ring = ogr.Geometry(ogr.wkbLinearRing)
             aCoords_gcs = po.exterior.coords
@@ -362,18 +340,7 @@ def calculate_area_of_difference_simplified(aFlowline_in, aVertex_all_in, \
             pFeature.SetField("id", lCellID)
             pLayer.CreateFeature(pFeature)
             lCellID= lCellID+1
-
             dArea0 = calculate_polygon_area(lons, lats)
             dArea = dArea + dArea0
 
-    return aPolygon_out, dArea
-   
-    
-
-    
-
-      
-
-             
-
-    
+    return aPolygon_out, dArea
\ No newline at end of file
