@@ -1,38 +1,14 @@
 import copy
 import numpy as np
 
-def find_vertex_in_list(aVertex_in, pVertex_in):
-    """[find the index of a vertex in a list]
 
-    Args:
-        aVertex_in ([type]): [description]
-        pVertex_in ([type]): [description]
-
-    Returns:
-        [type]: [description]
-    """
-
-    iFlag_exist = 0
-    lIndex= -1
-    nVertex= len(aVertex_in)
-
-    if nVertex > 0 :
-        for i in np.arange( nVertex):
-            pVertex = aVertex_in[i]
-            if pVertex == pVertex_in:
-                iFlag_exist = 1      
-                lIndex = i 
-                break                
-            else:
-                pass
-
-        pass        
-        
-    else:
-        pass
-    
-    return iFlag_exist, lIndex
+import importlib
+iFlag_cython = importlib.util.find_spec("cython") 
+if iFlag_cython is not None:
+    from pyflowline.algorithms.cython.kernel import find_vertex_in_list
+else:
 
+    from pyflowline.algorithms.auxiliary.find_vertex_in_list import find_vertex_in_list
 
 
 def find_vertex_on_edge(aVertex_in, pEdge_in):
@@ -173,7 +149,7 @@ def check_if_duplicates(aList_in):
     return iFlag_unique
 
 
-def add_unique_vertex(aVertex_in, pVertex_in):
+def add_unique_vertex(aVertex_in, pVertex_in, dThreshold_in = 1.0E-6):
     """[add a vertex to a list if it is not already included]
 
     Args:
@@ -186,7 +162,7 @@ def add_unique_vertex(aVertex_in, pVertex_in):
     iFlag_exist = 0
     nVertex = len(aVertex_in)     
 
-    iFlag_exist, dummy =  find_vertex_in_list(aVertex_in, pVertex_in)
+    iFlag_exist, dummy =  find_vertex_in_list(aVertex_in, pVertex_in, dThreshold_in)
 
     if iFlag_exist == 1:
         pass
