@@ -1,14 +1,12 @@
-from abc import ABCMeta, abstractmethod
+
 
 import copy
-
 import json
 from json import JSONEncoder
 import numpy as np
 from pyflowline.classes.vertex import pyvertex
 from pyflowline.classes.edge import pyedge
 
-
 class FlowlineClassEncoder(JSONEncoder):
     def default(self, obj):
         if isinstance(obj, np.integer):
@@ -27,7 +25,14 @@ class FlowlineClassEncoder(JSONEncoder):
         return JSONEncoder.default(self, obj)
 
 class pyflowline(object):
-    __metaclass__ = ABCMeta 
+    """The pyflowline class
+
+    Args:
+        object (object): None
+
+    Returns:
+        pyflowline: The flowline object
+    """
 
     lFlowlineID=-1
     lIndex=-1
@@ -57,7 +62,13 @@ class pyflowline(object):
     aFlowlineID_end_start = None
     aFlowlineID_end_end = None
     
-    def __init__(self, aEdge):    
+    def __init__(self, aEdge):
+        """
+        Initilize a flowline object
+
+        Args:
+            aEdge (list [pyedge]): A list of edge objects
+        """
         self.aEdge = aEdge
         nEdge  = len(aEdge)
         self.nEdge = nEdge
@@ -80,21 +91,35 @@ class pyflowline(object):
         return
 
     def calculate_length(self):
+        """
+        Calcualte the length
+
+        Returns:
+            float: The length of the flowline
+        """
         dLength =0.0
         #loop though
-        for edge in self.aEdge:
-            edge.calculate_length()
-            dLength = dLength + edge.dLength
+        for i in range(self.nEdge):
+        #for edge in self.aEdge:
+            self.aEdge[i].calculate_length()
+            dLength = dLength + self.aEdge[i].dLength
 
         #assing
         self.dLength= dLength
 
         return dLength
 
-    
-    
     def check_upstream(self, other):
-        iFlag_upstream =-1
+        """
+        Check whether another flowline is upstream or not
+
+        Args:
+            other (pyflowline): The other flowline
+
+        Returns:
+            int: 1 if it is, 0 if not
+        """
+        iFlag_upstream = -1
         v0 = self.pVertex_start
         v1 = self.pVertex_end
 
@@ -109,6 +134,15 @@ class pyflowline(object):
         return iFlag_upstream
 
     def check_downstream(self, other):
+        """
+        Check whether another flowline is downstream or not 
+
+        Args:
+            other (pyflowline): The other flowline
+
+        Returns:
+            int: 1 if it is, 0 if not
+        """
         iFlag_downstream =-1
         v0 = self.pVertex_start
         v1 = self.pVertex_end
@@ -122,9 +156,9 @@ class pyflowline(object):
         return iFlag_downstream
     
     def reverse(self):
-        '''
-        reverse the direction of a flowline
-        '''
+        """
+        Reverse a flowline
+        """
         aVertex = self.aVertex 
         nVertex = self.nVertex
         aVertex_new = list()
@@ -145,6 +179,15 @@ class pyflowline(object):
         self.pVertex_end =  aEdge[ nEdge-1  ].pVertex_end
 
     def merge_upstream(self, other):
+        """
+        Merge two flowlines as one
+
+        Args:
+            other (pyflowline): The other flowline
+
+        Returns:
+            pyflowline: The merged flowline
+        """
         pFlowline_out = copy.deepcopy(other)    
         
         iFlag_dam1 = other.iFlag_dam
@@ -187,16 +230,57 @@ class pyflowline(object):
         else:
             pass
 
+        #is this necessary
+        #pFlowline_out.iStream_segment = self.iStream_segment
+
+        return pFlowline_out
+    
+    def split_by_length(self, dDistance):
+        """
+        Split a flowline using the length threshold
+
+        Args:
+            dDistance (float): The length threshold for each edge
+
+        Returns:
+            pyflowline: The updated flowline
+        """
+        aEdge=list()
+        pFlowline_out=None
+        for edge in self.aEdge:
+            edge.calculate_length()
+            if edge.dLength > dDistance:
+                #break it
+                aEdge0=edge.split_by_length(dDistance)
+                for edge0 in aEdge0:
+                    aEdge.append(edge0)
+                pass
+            else:
+                aEdge.append(edge)
+                pass
+        pFlowline_out=pyflowline(aEdge)
         return pFlowline_out
 
     def calculate_flowline_sinuosity(self):
+        """
+        Calculate the sinuosoty of a flowline
+        """
         pVertex_start = self.pVertex_start
         pVertex_end = self.pVertex_end
         dDistance = pVertex_start.calculate_distance(pVertex_end)
         self.dSinuosity = self.dLength / dDistance
         return
 
-    def __eq__(self, other):                       
+    def __eq__(self, other):
+        """
+        Check whether two flowline are equivalent
+
+        Args:
+            other (pyflowline): The other flowline
+
+        Returns:
+            int: 1 if equivalent, 0 if not
+        """
         iFlag_overlap = 0 
         nEdge1 = self.nEdge
         nEdge2 = other.nEdge
@@ -216,9 +300,24 @@ class pyflowline(object):
         return iFlag_overlap
 
     def __ne__(self, other):
+        """
+        Check whether two flowline are equivalent
+
+        Args:
+            other (pyflowline): The other flowline
+
+        Returns:
+            int: 0 if equivalent, 1 if not
+        """
         return not self.__eq__(other)
     
     def tojson(self):
+        """
+        Convert a pyflowline object to a json string
+
+        Returns:
+            json str: A json string
+        """
         aSkip = ['aEdge', \
                 'aVertex','aFlowlineID_start_start','aFlowlineID_start_end',
                 'aFlowlineID_end_start','aFlowlineID_end_end']
