@@ -1,8 +1,7 @@
 
 import json
-import numpy as np
 from json import JSONEncoder
-
+import numpy as np
 from pyflowline.classes.vertex import pyvertex
 from pyflowline.classes.edge import pyedge
 from pyflowline.classes.cell import pycell
@@ -24,17 +23,41 @@ class LinkClassEncoder(JSONEncoder):
         return JSONEncoder.default(self, obj)
 
 class pycelllink(object):
+    """
+    The cell link class
+
+    Args:
+        object (obj): None
+
+    Returns:
+        pycelllink: A link object
+    """
+
     lIndex=0
     pCell_start=None
     pCell_end=None
     pEdge_link = None
     def __init__(self, pCell_start_in, pCell_end_in, pEdge_link_in):
+        """
+        Initilize a link object
+
+        Args:
+            pCell_start_in (pycell): The starting cell object
+            pCell_end_in (pycell): The ending cell object
+            pEdge_link_in (pyedge): An edge object that links two cells
+        """
         self.pCell_start = pCell_start_in
         self.pCell_end = pCell_end_in
         self.pEdge_link = pEdge_link_in
         return
     
-    def tojson(self):        
+    def tojson(self):
+        """
+        Convert a cell link object to a json string
+
+        Returns:
+            json str: A json string
+        """
         sJson = json.dumps(self.__dict__, \
             ensure_ascii=True, \
                 indent=4, \
