@@ -1,22 +1,18 @@
-
-from abc import ABCMeta, abstractmethod
+from abc import abstractmethod
 class mesh(object):
     """Abstract base class for
     """
-    aParameter = {}
+    
     dLatitude_bot = -90
     dLatitude_top = 90
     dLongitude_left = -180
     dLongitude_right = 180
-    aRectangle=[]
-    sFilename_mesh=''
-    __metaclass__ = ABCMeta    
+    sFilename_mesh=''   
     @abstractmethod
-    def __init__(self, aParameter):
+    def __init__(self):
 
         pass
     @abstractmethod
-   
     def calculate_mesh_area(self):
 
         pass
\ No newline at end of file
