@@ -1,13 +1,12 @@
 
-import numpy as np
+
 import json
 from json import JSONEncoder
-
+import numpy as np
 from pyflowline.classes.vertex import pyvertex
 from pyflowline.classes.edge import pyedge
 from pyflowline.classes.cell import pycell
 from pyflowline.classes.flowline import pyflowline
-
 from pyflowline.algorithms.auxiliary.gdal_functions import calculate_polygon_area
 
 class MpasClassEncoder(JSONEncoder):
@@ -35,7 +34,16 @@ class MpasClassEncoder(JSONEncoder):
 
 
 class pympas(pycell):
-    lCellID  = -1    
+    """
+    The MPAS cell class
+
+    Args:
+        pycell (object): None
+
+    Returns:
+        pympas: A mpas cell object
+    """
+    lCellID  = -1  
     nFlowline=0
     nVertex =0 
     nEdge=0
@@ -68,10 +76,20 @@ class pympas(pycell):
     aNeighbor_ocean=None #the global ID of all neighbors
     aNeighbor_distance = None
 
-    def __init__(self, dLon, dLat, aEdge, aVertex):    
+    def __init__(self, dLon, dLat, aEdge, aVertex):
+        """
+        Initilize a mpas cell object
+
+        Args:
+            dLon (float): The longitude of center 
+            dLat (float): The latitude of center 
+            aEdge (list [pyedge]): A list of edges that define the hexagon
+            aVertex (list [pyvertex]): A list of vertices the define the hexagon
+        """
+
         nEdge = len(aEdge)
-        if nEdge < 3 or nEdge > 8:
-            print('At lease 3 edges are required!')
+        if nEdge < 3 or nEdge > 9:
+            print('At lease 3 edges are required!', nEdge)
             pass
         else:                          
             self.aEdge = aEdge
@@ -98,6 +116,15 @@ class pympas(pycell):
     
    
     def has_this_edge(self, pEdge_in):
+        """
+        Check whether the cell contains an edge
+
+        Args:
+            pEdge_in (pyedge): the to be checked edge
+
+        Returns:
+            int: 1 if contains; or else 0
+        """
         iFlag_found = 0
         for pEdge in self.aEdge:
             if pEdge.is_overlap(pEdge_in):
@@ -109,6 +136,15 @@ class pympas(pycell):
         return iFlag_found
 
     def which_edge_cross_this_vertex(self, pVertex_in):
+        """
+        When a flowline intersects with a cell, this function finds out which edge is intersected
+
+        Args:
+            pVertex_in (pyvertex): the intersected vertex
+
+        Returns:
+            tuple: (1, edge) if contains; or else (0, None) 
+        """
         iFlag_found = 0
         pEdge_out = None
         for pEdge in self.aEdge:
@@ -122,7 +158,13 @@ class pympas(pycell):
 
         return iFlag_found, pEdge_out
     
-    def calculate_cell_area(self): #not used anymore
+    def calculate_cell_area(self):
+        """
+        Calculate the area of a cell, this function is not used for mpas cell
+
+        Returns:
+            float: cell area
+        """
         lons=list()
         lats=list()        
         for i in range(self.nVertex):            
@@ -132,11 +174,26 @@ class pympas(pycell):
         self.dArea = calculate_polygon_area(lons,lats )
         return self.dArea
 
-    def calculate_edge_length(self):        
+    def calculate_edge_length(self):
+        """
+        Calculate the effective cell length/resolution
+
+        Returns:
+            float: effective cell length/resolution
+        """
         self.dLength_edge = np.sqrt( self.dArea )
         return self.dLength_edge
     
     def share_edge(self, other):
+        """
+        Check if two cells share an edge
+
+        Args:
+            other (pympas): the other cell
+
+        Returns:
+            int: 1 if shared, 0 if not
+        """
         iFlag_share = 0
         for pEdge in self.aEdge:
             for pEdge2 in other.aEdge:
@@ -148,6 +205,12 @@ class pympas(pycell):
 
     
     def tojson(self):
+        """
+        Convert a cell into a json string
+
+        Returns:
+            json str: A json string
+        """
         aSkip = ['aEdge', \
                 'aFlowline']
         obj = self.__dict__.copy()
