@@ -1,11 +1,12 @@
 import os
 from pathlib import Path
-from abc import ABCMeta
 import json
 from json import JSONEncoder
 import datetime
+import importlib
 import numpy as np
 from osgeo import ogr, osr
+from shapely.wkt import loads
 from pyflowline.classes.mpas import pympas
 from pyflowline.classes.hexagon import pyhexagon
 from pyflowline.classes.latlon import pylatlon
@@ -14,11 +15,12 @@ from pyflowline.classes.vertex import pyvertex
 from pyflowline.classes.basin import pybasin
 from pyflowline.classes.flowline import pyflowline
 from pyflowline.classes.edge import pyedge
-from pyflowline.formats.read_mesh import read_mesh_json
+from pyflowline.formats.read_mesh import read_mesh_json, read_mesh_json_w_topology
 from pyflowline.algorithms.auxiliary.gdal_functions import reproject_coordinates
 from pyflowline.algorithms.auxiliary.gdal_functions  import degree_to_meter
 from pyflowline.algorithms.auxiliary.gdal_functions  import meter_to_degree
 from pyflowline.algorithms.auxiliary.gdal_functions import retrieve_geotiff_metadata
+from pyflowline.algorithms.auxiliary.gdal_functions import read_mesh_boundary
 from pyflowline.mesh.hexagon.create_hexagon_mesh import create_hexagon_mesh
 from pyflowline.mesh.latlon.create_latlon_mesh import create_latlon_mesh
 from pyflowline.mesh.square.create_square_mesh import create_square_mesh
@@ -58,31 +60,43 @@ class CaseClassEncoder(JSONEncoder):
         return JSONEncoder.default(self, obj)
 
 class flowlinecase(object):
-    __metaclass__ = ABCMeta
+    """
+    The flowline case class
+
+    Args:
+        object (obj): None
+
+    Returns:
+        flowlinecase: A flowlinecase object
+    """
     iCase_index= 0
+    
     sMesh_type = 1
     iFlag_standalone=1
     iFlag_flowline = 1
     iFlag_global = 0
     iFlag_multiple_outlet = 0
-    iFlag_use_shapefile_extent=1
+    iFlag_mesh_boundary = 0
+    #iFlag_use_shapefile_extent=1
     iFlag_use_mesh_dem=0
     iFlag_save_mesh = 0
     iFlag_simplification = 1 #user can turn on/off
     iFlag_create_mesh=1
     iFlag_intersect = 1
-    
     iFlag_rotation=0
-
+    iFlag_break_by_distance = 0  #if the distance between two vertice are far, 
     nOutlet = 1 #by default , there shoule ne only one ouelet
     dResolution_degree=0.0
     dResolution_meter=0.0
     dThreshold_small_river=0.0
+    dThreshold_break_by_distance = 5000.0 
     dLongitude_left = -180
     dLongitude_right = 180
     dLatitude_bot = -90
     dLatitude_top = 90
+    dElevation_mean=-9999.0
     sFilename_model_configuration=''
+    sFilename_mesh_boundary = ''
     sWorkspace_input=''      
     sWorkspace_output=''    
     #sWorkspace_output_case=''    
@@ -100,19 +114,39 @@ class flowlinecase(object):
     sFilename_mesh_info=''
     sFilename_mesh_netcdf=''
     
-
     aBasin = list()
     aFlowline_simplified=list()
     aFlowline_conceptual=list()
     aCellID_outlet = list()
     aCell=list()
 
+    iFlag_visual = importlib.util.find_spec("cartopy") 
+    if iFlag_visual is not None:
+        from ._visual import plot
+        from ._visual import _plot_study_area
+        from ._visual import _plot_mesh
+        from ._visual import _plot_mesh_with_flowline
+        from ._visual import _compare_with_raster_dem_method
+    else:
+        pass
+
+    from ._hpc import _create_hpc_job
     
-    def __init__(self, aConfig_in,\
-            iFlag_standalone_in= None,\
-            sModel_in = None,\
-            sDate_in = None,\
+    def __init__(self, aConfig_in,
+            iFlag_standalone_in= None,
+            sModel_in = None,
+            sDate_in = None,
             sWorkspace_output_in = None):
+        """
+        Initialize a flowlinecase object
+
+        Args:
+            aConfig_in (dict): A dictionary of parameters
+            iFlag_standalone_in (int, optional): Flag for whether run the case standalone. Defaults to None.
+            sModel_in (str, optional): The model name. Defaults to None.
+            sDate_in (str, optional): The case date. Defaults to None.
+            sWorkspace_output_in (str, optional): The output workspace. Defaults to None.
+        """
         #flags
         if iFlag_standalone_in is not None:
             self.iFlag_standalone = iFlag_standalone_in
@@ -131,6 +165,11 @@ class flowlinecase(object):
         
         if 'iFlag_global' in aConfig_in:
             self.iFlag_global             = int(aConfig_in[ 'iFlag_global'])
+
+        if 'iFlag_mesh_boundary' in aConfig_in:
+            self.iFlag_mesh_boundary             = int(aConfig_in[ 'iFlag_mesh_boundary'])
+        else:
+            self.iFlag_mesh_boundary=0
         
         if 'iFlag_multiple_outlet' in aConfig_in:
             self.iFlag_multiple_outlet             = int(aConfig_in[ 'iFlag_multiple_outlet'])  
@@ -155,8 +194,8 @@ class flowlinecase(object):
         if 'iFlag_use_mesh_dem' in aConfig_in:
             self.iFlag_use_mesh_dem = int(aConfig_in['iFlag_use_mesh_dem'])
 
-        if 'iFlag_use_shapefile_extent' in aConfig_in:
-            self.iFlag_use_shapefile_extent = int(aConfig_in['iFlag_use_shapefile_extent'])    
+        #if 'iFlag_use_shapefile_extent' in aConfig_in:
+        #    self.iFlag_use_shapefile_extent = int(aConfig_in['iFlag_use_shapefile_extent'])    
 
         if 'iFlag_rotation' in aConfig_in:
             self.iFlag_rotation = int(aConfig_in['iFlag_rotation'])
@@ -166,6 +205,11 @@ class flowlinecase(object):
             self.iFlag_intersect = int(aConfig_in['iFlag_intersect'])
         else:
             self.iFlag_intersect=1
+        
+        if 'iFlag_break_by_distance' in aConfig_in:
+            self.iFlag_break_by_distance = int(aConfig_in['iFlag_break_by_distance'])
+        else:
+            self.iFlag_break_by_distance=0
 
         if 'nOutlet' in aConfig_in:
             self.nOutlet = int(aConfig_in['nOutlet'])
@@ -185,6 +229,9 @@ class flowlinecase(object):
         else:
             print('Please specify resolution.')
 
+        if 'dThreshold_break_by_distance' in aConfig_in:
+            self.dThreshold_break_by_distance = float(aConfig_in['dThreshold_break_by_distance']) 
+
         if 'dLongitude_left' in aConfig_in:
             self.dLongitude_left = float(aConfig_in['dLongitude_left']) 
 
@@ -200,9 +247,19 @@ class flowlinecase(object):
         if 'sFilename_model_configuration' in aConfig_in:
             self.sFilename_model_configuration    = aConfig_in[ 'sFilename_model_configuration']
 
+        if 'sFilename_mesh_boundary' in aConfig_in:
+            self.sFilename_mesh_boundary    = aConfig_in[ 'sFilename_mesh_boundary']
+
+            if self.iFlag_mesh_boundary==1:
+                if not os.path.isfile(self.sFilename_mesh_boundary ):
+                    print("The mesh boundary file does not exist!")
+                    exit()
+                pass
+
         if 'sFilename_spatial_reference' in aConfig_in:
             self.sFilename_spatial_reference = aConfig_in['sFilename_spatial_reference']
 
+
         if 'sFilename_dem' in aConfig_in:
             self.sFilename_dem = aConfig_in['sFilename_dem']
 
@@ -213,8 +270,7 @@ class flowlinecase(object):
             self.sWorkspace_bin= aConfig_in[ 'sWorkspace_bin']
             
         if 'sWorkspace_input' in aConfig_in:
-            self.sWorkspace_input = aConfig_in[ 'sWorkspace_input']
-        
+            self.sWorkspace_input = aConfig_in[ 'sWorkspace_input']       
        
         
         if sWorkspace_output_in is not None:
@@ -281,6 +337,15 @@ class flowlinecase(object):
         
         Path(sPath).mkdir(parents=True, exist_ok=True)
 
+        if self.iMesh_type == 4:
+            if not os.path.isfile(self.sFilename_mesh_netcdf ):
+                print("The MPAS mesh file does not exist!")
+                exit()
+        else:
+            if not os.path.isfile(self.sFilename_dem ):
+                print("The DEM file does not exist!")
+                exit()
+
         self.aBasin = list()
     
         if self.iFlag_flowline==1:
@@ -294,12 +359,10 @@ class flowlinecase(object):
                 with open(self.sFilename_basins) as json_file:
                     dummy_data = json.load(json_file)     
                     for i in range(self.nOutlet):
-                        sBasin =  "{:03d}".format(i+1)   
+                        sBasin =  "{:08d}".format(i+1)   
                         dummy_basin = dummy_data[i]
-                        dummy_basin['sWorkspace_output_basin'] = str(Path(self.sWorkspace_output) /     sBasin )
-
+                        dummy_basin['sWorkspace_output_basin'] = str(Path(self.sWorkspace_output) / sBasin )
                         pBasin = pybasin(dummy_basin)
-
                         self.aBasin.append(pBasin)
             else:
                 pass
@@ -308,16 +371,16 @@ class flowlinecase(object):
      
 
         #model generated files         
-        self.sFilename_mesh = os.path.join(str(Path(self.sWorkspace_output)  ) , sMesh_type + ".json" )               
+        self.sFilename_mesh = os.path.join(str(Path(self.sWorkspace_output)  ) , sMesh_type + ".geojson" )               
         self.sFilename_mesh_info= os.path.join(str(Path(self.sWorkspace_output)  ) , sMesh_type + "_mesh_info.json"  )    
                 
         return
         
 
-    def convert_flowline_to_json(self):
+    def convert_flowline_to_geojson(self):
         if self.iFlag_flowline == 1:
             for pBasin in self.aBasin:            
-                pBasin.convert_flowline_to_json()
+                pBasin.convert_flowline_to_geojson()
                 pass
 
         return
@@ -327,7 +390,8 @@ class flowlinecase(object):
         #export the outlet into a single file
         aOutlet = list()
         if self.iFlag_simplification == 1: 
-            for pBasin in self.aBasin:
+            for i in range(self.nOutlet):
+                pBasin = self.aBasin[i]
                 aFlowline_basin = pBasin.flowline_simplification()                
                 aFlowline_out = aFlowline_out + aFlowline_basin
                 aOutlet.append(pBasin.pVertex_outlet)
@@ -337,41 +401,62 @@ class flowlinecase(object):
 
         return aFlowline_out
     
-    def mesh_generation(self):        
+    def mesh_generation(self):
+        """
+        The mesh generation operation
+
+        Returns:
+            list [pycell]: A list of cell object
+        """
+        print('Start mesh generation.')
+        aCell_out = list()  
         if self.iFlag_create_mesh ==1:
             iFlag_global =  self.iFlag_global
             iMesh_type = self.iMesh_type
             iFlag_save_mesh = self.iFlag_save_mesh
             iFlag_rotation = self.iFlag_rotation
+            iFlag_mesh_boundary = self.iFlag_mesh_boundary
             dResolution_degree = self.dResolution_degree
             dResolution_meter = self.dResolution_meter
             sFilename_dem = self.sFilename_dem
             sFilename_spatial_reference = self.sFilename_spatial_reference
             sFilename_mesh = self.sFilename_mesh
-            if iMesh_type !=4: #hexagon
+            if iMesh_type !=4: #mpas
                 spatial_reference_target = osr.SpatialReference()  
                 spatial_reference_target.ImportFromEPSG(4326)
-                if self.iFlag_use_shapefile_extent==1:
-                    pDriver_shapefile = ogr.GetDriverByName('Esri Shapefile')
-                    pDataset_shapefile = pDriver_shapefile.Open(self.sFilename_spatial_reference, 0)
-                    pLayer_shapefile = pDataset_shapefile.GetLayer(0)
-                    pSpatial_reference = pLayer_shapefile.GetSpatialRef()
-                    (dOriginX, dX_right, dY_bot, dOriginY) = pLayer_shapefile.GetExtent() # not in same order as ogrinfo
-                    dLongitude_left,  dLatitude_bot= reproject_coordinates(dOriginX, dY_bot,pSpatial_reference,    spatial_reference_target)
-                    dLongitude_right, dLatitude_top= reproject_coordinates(dX_right, dOriginY,pSpatial_reference,  spatial_reference_target)
-                    dLatitude_mean = 0.5 * (dLatitude_top + dLatitude_bot)
-                    pass
-                else:
-                    dPixelWidth, dOriginX, dOriginY, nrow, ncolumn, pSpatialRef_dem, pProjection, pGeotransform\
-                         = retrieve_geotiff_metadata(sFilename_dem)
+               
+                dPixelWidth, dOriginX, dOriginY, nrow, ncolumn, pSpatialRef_dem, pProjection, pGeotransform\
+                     = retrieve_geotiff_metadata(sFilename_dem)
 
-                    dY_bot = dOriginY - (nrow+1) * dPixelWidth
-                    dLongitude_left,  dLatitude_bot= reproject_coordinates(dOriginX, dY_bot,pSpatialRef_dem,    spatial_reference_target)
-                    dX_right = dOriginX + (ncolumn +1) * dPixelWidth
+                #lower left
+                dX_lowerleft  = dOriginX
+                dY_lowerleft = dOriginY - (nrow+1) * dPixelWidth
+                dLongitude_left0,  dLatitude_bot0= reproject_coordinates(dX_lowerleft, dY_lowerleft,pSpatialRef_dem,    spatial_reference_target)
+                
+                #upper right
+                dX_upperright = dOriginX + (ncolumn +1) * dPixelWidth
+                dY_upperright = dOriginY
+                dLongitude_right0, dLatitude_top0= reproject_coordinates(dX_upperright, dY_upperright,pSpatialRef_dem,  spatial_reference_target)
+                
+                #lower right
+                dX_lowerright = dOriginX + (ncolumn +1) * dPixelWidth
+                dY_lowerright = dOriginY - (nrow+1) * dPixelWidth
+                
+                dLongitude_right1,  dLatitude_bot1= reproject_coordinates(dX_lowerright, dY_lowerright,pSpatialRef_dem,    spatial_reference_target)
+                
+                #uppler left     
+                dX_upperleft   = dOriginX
+                dY_upperleft   =  dOriginY
+                dLongitude_left1, dLatitude_top1= reproject_coordinates(dX_upperleft, dY_upperleft,pSpatialRef_dem,  spatial_reference_target)
+                
+                dLatitude_top = np.max([dLatitude_top0, dLatitude_top1])
+                dLatitude_bot = np.min([dLatitude_bot0, dLatitude_bot1])
 
-                    dLongitude_right, dLatitude_top= reproject_coordinates(dX_right, dOriginY,pSpatialRef_dem,  spatial_reference_target)
-                    dLatitude_mean = 0.5 * (dLatitude_top + dLatitude_bot)
-                    pass
+                dLongitude_left = np.min([dLongitude_left0, dLongitude_left1])
+                dLongitude_right = np.max([dLongitude_right0, dLongitude_right1])
+
+                dLatitude_mean = 0.5 * (dLatitude_top + dLatitude_bot)
+                    #pass
 
                 if dResolution_meter < 0:
                     #not used
@@ -379,8 +464,8 @@ class flowlinecase(object):
                 else:
                     dResolution_degree = meter_to_degree(dResolution_meter, dLatitude_mean)
 
-                dX_left = dOriginX
-                dY_top = dOriginY
+                dX_lowerleft = dOriginX
+                dY_upperleft = dOriginY
             else:
                 pass
 
@@ -392,49 +477,128 @@ class flowlinecase(object):
                 if iFlag_rotation ==0:            
                     dX_spacing = dLength_edge * np.sqrt(3.0)
                     dY_spacing = dLength_edge * 1.5
-                    ncolumn= int( (dX_right - dX_left) / dX_spacing )
-                    nrow= int( (dY_top - dY_bot) / dY_spacing ) 
+                    ncolumn= int( (dX_lowerright - dX_lowerleft) / dX_spacing )
+                    nrow= int( (dY_upperleft - dY_lowerleft) / dY_spacing ) 
                 else:            
                     dX_spacing = dLength_edge * 1.5
                     dY_spacing = dLength_edge * np.sqrt(3.0)    
-                    ncolumn= int( (dX_right - dX_left) / dX_spacing )+1
-                    nrow= int( (dY_top - dY_bot) / dY_spacing )
+                    ncolumn= int( (dX_lowerright - dX_lowerleft) / dX_spacing )+1
+                    nrow= int( (dY_upperleft - dY_lowerleft) / dY_spacing )
+                
+                if iFlag_mesh_boundary ==1:
+                    #create a polygon based on real boundary 
+                    pBoundary = read_mesh_boundary(self.sFilename_mesh_boundary)
+                    
+                    aHexagon = create_hexagon_mesh(iFlag_rotation, dX_lowerleft, dY_lowerleft, dResolution_meter, ncolumn, nrow, pBoundary,\
+                        sFilename_mesh, sFilename_spatial_reference)
+                    pass
+                else:
+                    pRing = ogr.Geometry(ogr.wkbLinearRing)
+                    pRing.AddPoint(dLongitude_left, dLatitude_top)
+                    pRing.AddPoint(dLongitude_right, dLatitude_top)
+                    pRing.AddPoint(dLongitude_right, dLatitude_bot)
+                    pRing.AddPoint(dLongitude_left, dLatitude_bot)
+                    pRing.AddPoint(dLongitude_left, dLatitude_top)
+                    pBoundary = ogr.Geometry(ogr.wkbPolygon)
+                    pBoundary.AddGeometry(pRing)
+                    pBoundary_rec = loads( pBoundary.ExportToWkt() )
+
+                    aHexagon = create_hexagon_mesh(iFlag_rotation, dX_lowerleft, dY_lowerleft, dResolution_meter, ncolumn, nrow, pBoundary_rec,\
+                        sFilename_mesh, sFilename_spatial_reference)
 
-                aHexagon = create_hexagon_mesh(iFlag_rotation, dX_left, dY_bot, dResolution_meter, ncolumn, nrow, \
-                    sFilename_mesh, sFilename_spatial_reference)
                 return aHexagon
             else:
                 if iMesh_type ==2: #sqaure
-                    ncolumn= int( (dX_right - dX_left) / dResolution_meter )
-                    nrow= int( (dY_top - dY_bot) / dResolution_meter )
+                    ncolumn= int( (dX_lowerright - dX_lowerleft) / dResolution_meter )
+                    nrow= int( (dY_upperleft - dY_lowerleft) / dResolution_meter )
+                    if iFlag_mesh_boundary ==1:
+                        #create a polygon based on real boundary 
+                        pBoundary = read_mesh_boundary(self.sFilename_mesh_boundary)
 
-                    aSquare = create_square_mesh(dX_left, dY_bot, dResolution_meter, ncolumn, nrow, \
-                        sFilename_mesh, sFilename_spatial_reference)
-                    return aSquare
+                        aSquare = create_square_mesh(dX_lowerleft, dY_lowerleft, dResolution_meter, ncolumn, nrow, pBoundary ,\
+                            sFilename_mesh, sFilename_spatial_reference)
+                        pass
+                    else:
+                        pRing = ogr.Geometry(ogr.wkbLinearRing)
+                        pRing.AddPoint(dLongitude_left, dLatitude_top)
+                        pRing.AddPoint(dLongitude_right, dLatitude_top)
+                        pRing.AddPoint(dLongitude_right, dLatitude_bot)
+                        pRing.AddPoint(dLongitude_left, dLatitude_bot)
+                        pRing.AddPoint(dLongitude_left, dLatitude_top)
+                        pBoundary = ogr.Geometry(ogr.wkbPolygon)
+                        pBoundary.AddGeometry(pRing)
+                        pBoundary_rec = loads( pBoundary.ExportToWkt() )
+                        aSquare = create_square_mesh(dX_lowerleft, dY_lowerleft, dResolution_meter, ncolumn, nrow, pBoundary_rec, \
+                            sFilename_mesh, sFilename_spatial_reference)
+                        return aSquare
                 else:
                     if iMesh_type ==3: #latlon
                         dResolution_meter = degree_to_meter(dLatitude_mean, dResolution_degree)
                         dArea = np.power(dResolution_meter,2.0)
-                        dLatitude_top    = self.dLatitude_top   
-                        dLatitude_bot    = self.dLatitude_bot   
-                        dLongitude_left  = self.dLongitude_left 
-                        dLongitude_right = self.dLongitude_right
                         ncolumn= int( (dLongitude_right - dLongitude_left) / dResolution_degree )
                         nrow= int( (dLatitude_top - dLatitude_bot) / dResolution_degree )
-                        aLatlon = create_latlon_mesh(dLongitude_left, dLatitude_bot, dResolution_degree, ncolumn, nrow, \
-                            sFilename_mesh)
+                         
+                        if iFlag_mesh_boundary ==1:
+                            #create a polygon based on real boundary 
+                            pBoundary = read_mesh_boundary(self.sFilename_mesh_boundary)
+                            aLatlon = create_latlon_mesh(dLongitude_left, dLatitude_bot, dResolution_degree, ncolumn, nrow,pBoundary, \
+                                    sFilename_mesh)
+                            pass
+                        else:
+                            pRing = ogr.Geometry(ogr.wkbLinearRing)
+                            pRing.AddPoint(dLongitude_left, dLatitude_top)
+                            pRing.AddPoint(dLongitude_right, dLatitude_top)
+                            pRing.AddPoint(dLongitude_right, dLatitude_bot)
+                            pRing.AddPoint(dLongitude_left, dLatitude_bot)
+                            pRing.AddPoint(dLongitude_left, dLatitude_top)
+                            pBoundary = ogr.Geometry(ogr.wkbPolygon)
+                            pBoundary.AddGeometry(pRing)
+                            pBoundary_rec = loads( pBoundary.ExportToWkt() )
+                            aLatlon = create_latlon_mesh(dLongitude_left, dLatitude_bot, dResolution_degree, ncolumn, nrow,pBoundary_rec, \
+                                    sFilename_mesh)
+                            
+                            pass
+
                         return aLatlon
+                            
+
+                        
                     else:
                         if iMesh_type == 4: #mpas
                             iFlag_use_mesh_dem = self.iFlag_use_mesh_dem
-                            sFilename_mesh_netcdf = self.sFilename_mesh_netcdf
+                            sFilename_mesh_netcdf = self.sFilename_mesh_netcdf                            
                             dLatitude_top    = self.dLatitude_top   
                             dLatitude_bot    = self.dLatitude_bot   
                             dLongitude_left  = self.dLongitude_left 
                             dLongitude_right = self.dLongitude_right
-                            aMpas = create_mpas_mesh(iFlag_global, iFlag_use_mesh_dem, iFlag_save_mesh, \
-                                  dLongitude_left, dLongitude_right,  dLatitude_top, dLatitude_bot, \
-                                        sFilename_mesh_netcdf,      sFilename_mesh)
+
+                            if iFlag_mesh_boundary ==1:
+                                #create a polygon based on 
+                                #read boundary 
+                                pBoundary = read_mesh_boundary(self.sFilename_mesh_boundary)
+
+                                aMpas = create_mpas_mesh(iFlag_global, iFlag_use_mesh_dem, iFlag_save_mesh, \
+                                  pBoundary,       sFilename_mesh_netcdf,      sFilename_mesh)
+                                pass
+                            else:
+                                pRing = ogr.Geometry(ogr.wkbLinearRing)
+                                pRing.AddPoint(dLongitude_left, dLatitude_top)
+                                pRing.AddPoint(dLongitude_right, dLatitude_top)
+                                pRing.AddPoint(dLongitude_right, dLatitude_bot)
+                                pRing.AddPoint(dLongitude_left, dLatitude_bot)
+                                pRing.AddPoint(dLongitude_left, dLatitude_top)
+                                pBoundary = ogr.Geometry(ogr.wkbPolygon)
+                                pBoundary.AddGeometry(pRing)
+                                pBoundary_rec = loads( pBoundary.ExportToWkt() )
+
+                                #old method using rectange
+                                #aMpas = create_mpas_mesh(iFlag_global, iFlag_use_mesh_dem, iFlag_save_mesh, \
+                                #  dLongitude_left, dLongitude_right,  dLatitude_top, dLatitude_bot, \
+                                #        sFilename_mesh_netcdf,      sFilename_mesh)
+
+                                #new method using polygon object
+                                aMpas = create_mpas_mesh(iFlag_global, iFlag_use_mesh_dem, iFlag_save_mesh, \
+                                  pBoundary_rec,       sFilename_mesh_netcdf,      sFilename_mesh)
                             return aMpas
                         else:
                             if iMesh_type ==5: #tin this one need to be updated because central location issue
@@ -445,18 +609,32 @@ class flowlinecase(object):
                                 dY_shift = 0.5 * dLength_edge * np.sqrt(3.0) 
                                 dX_spacing = dX_shift * 2
                                 dY_spacing = dY_shift
-                                ncolumn= int( (dX_right - dX_left) / dX_shift )
-                                nrow= int( (dY_top - dY_bot) / dY_spacing ) 
-                                aTin = create_tin_mesh(dX_left, dY_bot, dResolution_meter, ncolumn, nrow,sFilename_mesh, sFilename_spatial_reference)
+                                ncolumn= int( (dX_lowerright - dX_lowerleft) / dX_shift )
+                                nrow= int( (dY_upperleft - dY_lowerleft) / dY_spacing ) 
+                                aTin = create_tin_mesh(dX_lowerleft, dY_lowerleft, dResolution_meter, ncolumn, nrow,sFilename_mesh, sFilename_spatial_reference)
                                 return aTin
                             else:
                                 print('Unsupported mesh type?')
                                 return
         else:
+            #read mesh?
+            iMesh_type = self.iMesh_type
+            aCell_out = read_mesh_json_w_topology(iMesh_type, self.sFilename_mesh)
             pass
-        return
+        print('Finish mesh generation.')
+        return aCell_out
     
     def reconstruct_topological_relationship(self, aCell_raw):
+        """
+        The topological relationship reconstruction operation
+
+        Args:
+            aCell_raw (list [pycell]): A list of intersected cell objects
+
+        Returns:
+            tuple [list [pycell], list [pyflowline], list [long]]: A list of cells, flowlines, and outlet cell IDs.
+        """
+        print('Start topology reconstruction.')
         iFlag_intersect = self.iFlag_intersect
         if iFlag_intersect == 1:
             iMesh_type = self.iMesh_type        
@@ -469,7 +647,7 @@ class flowlinecase(object):
             aCellID_outlet = list()
             aBasin = list()
             aCell_intersect=list()
-        
+            ncell=len(self.aCell)
             for pBasin in self.aBasin:
                 aCell_intersect_basin = pBasin.reconstruct_topological_relationship(iMesh_type,sFilename_mesh)
                 aFlowline_conceptual = aFlowline_conceptual + pBasin.aFlowline_basin_conceptual
@@ -477,6 +655,45 @@ class flowlinecase(object):
                 aCellID_outlet.append(pBasin.lCellID_outlet)
                 aCell_intersect = aCell_intersect + aCell_intersect_basin
 
+                #set topology here
+                nFlowline = len(pBasin.aFlowline_basin_conceptual)
+                for i in range(nFlowline):
+                    pFlowline = pBasin.aFlowline_basin_conceptual[i]
+                    nEdge = pFlowline.nEdge
+                    nVertex = pFlowline.nVertex
+                    aEdge = pFlowline.aEdge
+                    iStream_segment = pFlowline.iStream_segment
+                    iStream_order = pFlowline.iStream_order
+                    for j in range(nEdge):
+                        try:
+                            pEdge = aEdge[j]
+                            pVertex_start = pEdge.pVertex_start
+                            pVertex_end = pEdge.pVertex_end
+                            for k in range(ncell):
+                                pVertex_center = self.aCell[k].pVertex_center
+                                if pVertex_center == pVertex_start:
+                                    self.aCell[k].iStream_segment_burned = iStream_segment
+                                    self.aCell[k].iStream_order_burned = iStream_order
+                                    for l in range(ncell):
+                                        pVertex_center2 = self.aCell[l].pVertex_center
+                                        lCellID = self.aCell[l].lCellID
+                                        if pVertex_center2 == pVertex_end:
+                                            self.aCell[k].lCellID_downstream_burned = lCellID
+                                            if lCellID ==  pBasin.lCellID_outlet:
+                                                self.aCell[l].iStream_segment_burned = iStream_segment
+                                                self.aCell[l].iStream_order_burned = iStream_order
+
+                                            break
+                        except:
+                            print("error in step")
+                            print(pFlowline.tojson())
+                            print(pEdge.tojson())
+                            print(pVertex_start.tojson())
+                            print(pVertex_end.tojson())
+                            pass
+
+
+
                 #update length?
             for pCell in self.aCell:
                 for pCell2 in aCell_intersect:
@@ -485,15 +702,22 @@ class flowlinecase(object):
             
             self.aFlowline_conceptual = aFlowline_conceptual
             self.aCellID_outlet = aCellID_outlet
-
+            print('Finish topology reconstruction.')
             return self.aCell, aFlowline_conceptual, aCellID_outlet
         else:
+            
             return None
             
-
-        
-   
     def merge_cell_info(self, aCell_raw):
+        """
+        Merge cell information after reconstruction
+
+        Args:
+            aCell_raw (list [pycell]): The original cell information that contains neighbor definition
+
+        Returns:
+            list [pycell]: The updated list of cell objects.
+        """
 
         for pCell in self.aCell:
             for pCell2 in aCell_raw:
@@ -509,21 +733,38 @@ class flowlinecase(object):
         return self.aCell
         
     def analyze(self):
+        """
+        Analyze the domain results for every watershed
+        """
         if self.iFlag_flowline == 1:
             for pBasin in self.aBasin:
                 pBasin.analyze()
         return
 
     def setup(self):
+        """
+        Set up the flowlinecase
+        """
         if self.iFlag_flowline == 1:
-            self.convert_flowline_to_json()
+            self.convert_flowline_to_geojson()
+            pass
         return
 
     def run(self):
+        """
+        Run the flowlinecase simulation
+
+        Returns:
+            list: A list of cell objects
+        """
+        aCell_out = None
         if self.iFlag_flowline == 1:
             self.flowline_simplification()        
             aCell = self.mesh_generation()
-            aCell_out, a, b = self.reconstruct_topological_relationship(aCell)
+            if self.iFlag_intersect ==1:
+                aCell_out, a, b = self.reconstruct_topological_relationship(aCell)
+            else:
+                pass
         else:
             #only mesh generator
             aCell = self.mesh_generation()
@@ -532,60 +773,40 @@ class flowlinecase(object):
         
         return aCell_out
 
+    def evaluate(self):
+        """
+        Evaluate the model performance
+        """
+        for pBasin in self.aBasin:
+            pBasin.evaluate(self.iMesh_type, self.sMesh_type)
+        return
    
     def export(self):
-        
+        """
+        Export the model outputs
+        """        
         self.export_mesh_info_to_json()
         if self.iFlag_flowline ==1:
             for pBasin in self.aBasin:
                 pBasin.export()
 
         self.tojson()
+        return
+
+    def export_mesh_info_to_json(self):
+        """
+        Export the mesh information to a json file
+        """
 
-    def export_mesh_info_to_json(self):        
         aCell_all = self.aCell
         sFilename_json = self.sFilename_mesh_info
         ncell=len(aCell_all)
         iFlag_flowline = self.iFlag_flowline 
-        if iFlag_flowline == 1: #if there is conceptual flowline 
-            aCellID_outlet = self.aCellID_outlet
-            aFlowline = self.aFlowline_conceptual
-            nFlowline = len(aFlowline)
-            for i in range(nFlowline):
-                pFlowline = aFlowline[i]
-                nEdge = pFlowline.nEdge
-                nVertex = pFlowline.nVertex
-                aEdge = pFlowline.aEdge
-                iStream_segment = pFlowline.iStream_segment
-                iStream_order = pFlowline.iStream_order
-                for j in range(nEdge):
-                    pEdge = aEdge[j]
-                    pVertex_start = pEdge.pVertex_start
-                    pVertex_end = pEdge.pVertex_end
-                    for k in range(ncell):
-                        pVertex_center = aCell_all[k].pVertex_center
-
-                        if pVertex_center == pVertex_start:
-                            aCell_all[k].iStream_segment_burned = iStream_segment
-                            aCell_all[k].iStream_order_burned = iStream_order
-
-                            for l in range(ncell):
-                                pVertex_center2 = aCell_all[l].pVertex_center
-                                lCellID = aCell_all[l].lCellID
-                                if pVertex_center2 == pVertex_end:
-                                    aCell_all[k].lCellID_downstream_burned = lCellID
-                                    if lCellID in aCellID_outlet:
-                                        aCell_all[l].iStream_segment_burned = iStream_segment
-                                        aCell_all[l].iStream_order_burned = iStream_order
-
-                                    break
-                                
-                                
-
-                pass
-        else:
-            #only mesh, no flowline
-            pass
+        #if iFlag_flowline == 1: #if there is conceptual flowline 
+        #    pass
+        #else:
+        #    #only mesh, no flowline
+        #    pass
         
 
         with open(sFilename_json, 'w', encoding='utf-8') as f:
@@ -595,7 +816,13 @@ class flowlinecase(object):
 
         return
 
-    def tojson(self):  
+    def tojson(self): 
+        """
+        Convert the flowline case object to a json string
+
+        Returns:
+            json str: A json string
+        """
         aSkip = ['aBasin', \
                 'aFlowline_simplified','aFlowline_conceptual','aCellID_outlet',
                 'aCell']      
@@ -611,6 +838,12 @@ class flowlinecase(object):
         return sJson
 
     def export_config_to_json(self, sFilename_output_in = None):
+        """
+        Export the configuration to a json file
+
+        Args:
+            sFilename_output_in (str, optional): The json filename. Defaults to None.
+        """
 
         if self.iFlag_standalone == 1:
             if sFilename_output_in is not None:
@@ -668,8 +901,13 @@ class flowlinecase(object):
                 cls=CaseClassEncoder)
         return
 
-
     def export_basin_config_to_json(self, sFilename_output_in= None):
+        """
+        Export the member basin configuration to a json file
+
+        Args:
+            sFilename_output_in (str, optional): The json filename. Defaults to None.
+        """
         if self.iFlag_standalone == 1:
             if sFilename_output_in is not None:
                 sFilename_output = sFilename_output_in
@@ -707,4 +945,6 @@ class flowlinecase(object):
             #update for pyhexwatershed
             self.sFilename_basins = sFilename_output
             
-        return
\ No newline at end of file
+        return
+
+    
\ No newline at end of file
