@@ -1,8 +1,8 @@
 import os 
+from pathlib import Path
 import datetime
 import json
 from pyflowline.classes.pycase import flowlinecase
-
 pDate = datetime.datetime.today()
 sDate_default = "{:04d}".format(pDate.year) + "{:02d}".format(pDate.month) + "{:02d}".format(pDate.day)
 
@@ -91,7 +91,15 @@ def pyflowline_read_model_configuration_file(sFilename_configuration_in,   \
         sWorkspace_output = sWorkspace_output_in
     else:
         sWorkspace_output = aConfig['sWorkspace_output']
-        pass
+        #try to create this output folder first using 
+    
+    try:
+        print(sWorkspace_output)
+        Path(sWorkspace_output).mkdir(parents=True, exist_ok=True)
+    except ValueError:
+        print("The specified output workspace cannot be created!")
+        exit
+    
 
     
     aConfig['iCase_index'] = iCase_index
@@ -103,6 +111,8 @@ def pyflowline_read_model_configuration_file(sFilename_configuration_in,   \
     aConfig['sDate'] = sDate
     aConfig['sModel'] = sModel
     aConfig['sWorkspace_output'] = sWorkspace_output
+
+    aConfig["sFilename_model_configuration"] = sFilename_configuration_in
    
     
 
